#include "stdafx.h"
#include "utils.h"
#include "config.h"
#include "directinput.h"
#include "extern.h"

int initbeep = 0;
bool bgmode = false;

static const char * buttonNames[14] = {
	"A",
	"B",
	"X",
	"Y",
	"Left Shoulder",
	"Right Shoulder",
	"Back",
	"Start",
	"Left Thumb",
	"Right Thumb",
	"D-Pad Up",
	"D-Pad Down",
	"D-Pad Left",
	"D-Pad Right"
};
static const char * const axisNames[8] = {
	"Left Analog Right",
	"Left Analog Left",
	"Left Analog Up",
	"Left Analog Down",
	"Right Analog Right",
	"Right Analog Left",
	"Right Analog Up",
	"Right Analog Down"
};

static const char * triggerNames[2] = {
	"Left Trigger",
	"Right Trigger"
};

struct DIKey {
	const char * name;
	unsigned int key;
};

const DIKey KeyList[] = {
	{ "A", DIK_A },
	{ "B", DIK_B },
	{ "C", DIK_C },
	{ "D", DIK_D },
	{ "E", DIK_E },
	{ "F", DIK_F },
	{ "G", DIK_G },
	{ "H", DIK_H },
	{ "I", DIK_I },
	{ "J", DIK_J },
	{ "K", DIK_K },
	{ "L", DIK_L },
	{ "M", DIK_M },
	{ "N", DIK_N },
	{ "O", DIK_O },
	{ "P", DIK_P },
	{ "Q", DIK_Q },
	{ "R", DIK_R },
	{ "S", DIK_S },
	{ "T", DIK_T },
	{ "U", DIK_U },
	{ "V", DIK_V },
	{ "W", DIK_W },
	{ "X", DIK_X },
	{ "Y", DIK_Y },
	{ "Z", DIK_Z },
	{ "0", DIK_0 },
	{ "1", DIK_1 },
	{ "2", DIK_2 },
	{ "3", DIK_3 },
	{ "4", DIK_4 },
	{ "5", DIK_5 },
	{ "6", DIK_6 },
	{ "7", DIK_7 },
	{ "8", DIK_8 },
	{ "9", DIK_9 },
	{ "Num0", DIK_NUMPAD0 },
	{ "Num1", DIK_NUMPAD1 },
	{ "Num2", DIK_NUMPAD2 },
	{ "Num3", DIK_NUMPAD3 },
	{ "Num4", DIK_NUMPAD4 },
	{ "Num5", DIK_NUMPAD5 },
	{ "Num6", DIK_NUMPAD6 },
	{ "Num7", DIK_NUMPAD7 },
	{ "Num8", DIK_NUMPAD8 },
	{ "Num9", DIK_NUMPAD9 },
	{ "NumLock", DIK_NUMLOCK },
	{ "NumEnter", DIK_NUMPADENTER },
	{ "NumMinus", DIK_NUMPADMINUS },
	{ "NumPeriod", DIK_NUMPADPERIOD },
	{ "NumPlus", DIK_NUMPADPLUS },
	{ "NumSlash", DIK_NUMPADSLASH },
	{ "NumStar", DIK_NUMPADSTAR },
	{ "Up", DIK_UP },
	{ "Down", DIK_DOWN },
	{ "Left", DIK_LEFT },
	{ "Right", DIK_RIGHT },
	{ "F1", DIK_F1 },
	{ "F2", DIK_F2 },
	{ "F3", DIK_F3 },
	{ "F4", DIK_F4 },
	{ "F5", DIK_F5 },
	{ "F6", DIK_F6 },
	{ "F7", DIK_F7 },
	{ "F8", DIK_F8 },
	{ "F9", DIK_F9 },
	{ "F10", DIK_F10 },
	{ "F11", DIK_F11 },
	{ "F12", DIK_F12 },
	{ "Esc", DIK_ESCAPE },
	{ "Grave", DIK_GRAVE },
	{ "Minus", DIK_MINUS },
	{ "Equals", DIK_EQUALS },
	{ "Backspace", DIK_BACKSPACE },
	{ "Tab", DIK_TAB },
	{ "Left Bracket", DIK_LBRACKET },
	{ "Right Bracket", DIK_RBRACKET },
	{ "CapsLock", DIK_CAPSLOCK },
	{ "Semicolon", DIK_SEMICOLON },
	{ "Apostrophe", DIK_APOSTROPHE },
	{ "Backslash", DIK_BACKSLASH },
	{ "Return", DIK_RETURN },
	{ "Left Shift", DIK_LSHIFT },
	{ "Right Shift", DIK_RSHIFT },
	{ "Comma", DIK_COMMA },
	{ "Period", DIK_PERIOD },
	{ "Slash", DIK_SLASH },
	{ "Left Control", DIK_LCONTROL },
	{ "Right Control", DIK_RCONTROL },
	{ "Left Alt", DIK_LALT },
	{ "Right Alt", DIK_RALT },
	{ "Insert", DIK_INSERT },
	{ "Delete", DIK_DELETE },
	{ "Home", DIK_HOME },
	{ "End", DIK_END },
	{ "PgUp", DIK_PGUP },
	{ "PgDn", DIK_PGDN },
	{ "Space", DIK_SPACE },
	{ "SysRq", DIK_SYSRQ },
	{ "Scroll Lock", DIK_SCROLL },
	{ "Pause", DIK_PAUSE },
	{ NULL, -1 }
};

GamepadMap GamepadMapping[4];

void InitConfig() {

	char	*pStr, strPath[MAX_PATH];
	extern char strMySystemFile[MAX_PATH];	
	GetModuleFileName (NULL, strPath, MAX_PATH);
	pStr = strrchr(strPath, '\\');
	if (pStr != NULL)
		*(++pStr)='\0'; 

	char buffer[MAX_PATH];
	int padcount;

	sprintf_s(strMySystemFile,"%s%s",strPath, "x360kb.ini");

	// Read global options
	ReadFromFile("Options", "UseInitBeep", buffer, "1");	initbeep = atoi(buffer);
	ReadFromFile("Options", "Log", buffer, "0");			writelog = atoi(buffer);
	ReadFromFile("Options", "PadCount", buffer, "0");		padcount = atoi(buffer);
	ReadFromFile("Options", "BackgroundMode", buffer, "0");	bgmode = (bool) atoi(buffer);

	if ((padcount > 4) || (padcount < 1)) return;
	// Read pad mappings
	for (int i = 0; i < padcount; ++i) ReadPadConfig(i);

}

int GetKey(const char * name) {
	for (int i = 0; KeyList[i].name; ++i) {
		if (stricmp(name, KeyList[i].name) == 0) return KeyList[i].key;
	}
	return -1;
}

void ReadPadConfig(unsigned int idx) {

	char section[20];
	sprintf(section,"PAD%d",idx+1);
	GamepadMap &PadMap = GamepadMapping[idx];

	char buffer[MAX_PATH];

	PadMap.enabled = true;

	// Read buttons
	for (int i=0; i<14; ++i) {
		if (ReadFromFile(section, buttonNames[i], buffer) > 0) {
			PadMap.Button[i] = GetKey(buffer);
		}
	}
	// Read thumbsticks
	for (int i=0; i<8; ++i) {
		if (ReadFromFile(section, axisNames[i], buffer) > 0) {
			PadMap.Axis[i] = GetKey(buffer);			
		}
	}
	// Read triggers
	for (int i=0; i<2; ++i) {
		if (ReadFromFile(section, triggerNames[i], buffer) > 0) {
			PadMap.Trigger[i] = GetKey(buffer);
		}
	}
}
