#include "stdafx.h"
#include "extern.h"
#include "directinput.h"
#include "utils.h"

LPDIRECTINPUT8	g_pDI = NULL ;   //only one DirectInput intrface is needed
LPDIRECTINPUTDEVICE8	g_lpDIKeyboard = NULL; 

HWND hHookWnd = NULL;

VOID FreeDirectInput()
{
    // Unacquire the device one last time just in case 
    // the app tried to exit while the device is still acquired.
	if (g_lpDIKeyboard) {
		g_lpDIKeyboard->Unacquire();
		g_lpDIKeyboard->Release();
		g_lpDIKeyboard = NULL;
	}
	if ( g_pDI ) {
		g_pDI->Release();
		g_pDI = NULL;
	}
}

bool PollKeyboard( BYTE * buffer)
{
    HRESULT  hr; 
    hr = g_lpDIKeyboard->GetDeviceState(256,buffer); 
    if FAILED(hr) 
    {
		WriteLog("Failed to get device state, HR = %s", DXErrStr(hr));

		while ( hr == DIERR_INPUTLOST || hr == DIERR_NOTACQUIRED)
			hr = g_lpDIKeyboard->Acquire();

		return false;
    }
	return true;
}

HRESULT InitDirectInput( HWND hDlg )
{
    HRESULT hr;
	DWORD mode = bgmode ? DISCL_BACKGROUND : DISCL_FOREGROUND;
  
    FreeDirectInput();

    // Create a DInput object
    if( FAILED( hr = DirectInput8Create( GetModuleHandle(NULL), DIRECTINPUT_VERSION, 
		IID_IDirectInput8, (VOID**)&g_pDI, NULL ) ) )
	{
		WriteLog("Failed to create DInput, HR = %s", DXErrStr(hr));
		return hr;
	}
        

	//Create keyboard
	if( FAILED( hr = g_pDI->CreateDevice(GUID_SysKeyboard, &g_lpDIKeyboard, NULL) ) )
	{
		WriteLog("Failed to create device, HR = %s", DXErrStr(hr));
		return hr;
	}
	
	// Set keyboard data format
	if( FAILED( hr = g_lpDIKeyboard->SetDataFormat(&c_dfDIKeyboard) ) )
	{
		WriteLog("Failed to set data format, HR = %s", DXErrStr(hr));
		return hr;
	}

	if( FAILED( hr = g_lpDIKeyboard->SetCooperativeLevel(hDlg, mode | DISCL_NONEXCLUSIVE) ) )
	{
		WriteLog("Failed to set cooperative level, HR = %s", DXErrStr(hr));
		return hr;
	}

	if( FAILED (hr = g_lpDIKeyboard->Acquire()))
	{
		WriteLog("Failed to acquire device, HR = %s", DXErrStr(hr));
		return hr;
	}

	WriteLog("DirectInput initialized successfully.");

	return S_OK;
}
