#include "stdafx.h"
#include "utils.h"
#include "extern.h"
#include "config.h"
#include "directinput.h"


bool didinit = false;
HWND hWin = NULL;
DWORD PID = NULL;

extern struct DIGAMEPAD Gamepad[4];
extern void Initialize(DWORD idx);
extern HRESULT UpdateState(HWND, int );
extern HRESULT InitDirectInput( HWND hook, int idx, int failsafe );
extern int fallback[4];
extern int ConfigPadCount();
extern int EnumPadCount();
extern HRESULT Enumerate(DWORD idx);


BOOL CALLBACK findWindowByPID(HWND hwnd, LPARAM targetPID) {

	DWORD wndPID;
	GetWindowThreadProcessId (hwnd, &wndPID);
	if (wndPID == targetPID)
	{
		hWin = hwnd;
		return false;
	}
	return true;
}

inline bool Initialize() 
{
	WriteLog("Init started...");

	// Find our window
	if (hWin == NULL) EnumWindows(findWindowByPID,GetCurrentProcessId());

	return !FAILED( InitDirectInput(hWin) );
}


inline bool KeyDown(BYTE* buffer, int key) {
	if (key < 0) return false;
	return buffer[key] & 0x80;
}

DWORD WINAPI XInputGetState(DWORD dwUserIndex, XINPUT_STATE* pState)
{
	// Lazy initialization
	if (!didinit) {
		didinit = Initialize();
	}

	if(!GamepadMapping[dwUserIndex].enabled) return ERROR_DEVICE_NOT_CONNECTED;

	GamepadMap PadMap = GamepadMapping[dwUserIndex];
	XINPUT_GAMEPAD &Gamepad = pState->Gamepad;
	Gamepad.wButtons = 0;
	Gamepad.bLeftTrigger = 0;
	Gamepad.bRightTrigger = 0;
	Gamepad.sThumbLX = 0;
	Gamepad.sThumbLY = 0;
	Gamepad.sThumbRX = 0;
	Gamepad.sThumbRY = 0;

	// Timestamp packet
	pState->dwPacketNumber=GetTickCount();

	// Poll data from device
	BYTE diks[256];
    ZeroMemory( diks, sizeof(diks) );
	if (!PollKeyboard(diks)) return ERROR_SUCCESS;

	// Map buttons
	for (int i = 0; i < 14; ++i) {
		if (KeyDown(diks,PadMap.Button[i])) Gamepad.wButtons |= buttonIDs[i];
	}
	// Map thumbsticks
	// Created so we can refer to each axis with an ID
	SHORT *targetAxis[4] = {
		&Gamepad.sThumbLX,
		&Gamepad.sThumbLY,
		&Gamepad.sThumbRX,
		&Gamepad.sThumbRY
	};

	for (int i = 0; i < 8; ++i) {
		if (KeyDown(diks, PadMap.Axis[i])) {
			int val = (i % 2) ?  -32768 : 32767;
			*(targetAxis[i/2]) = val;
		}
	}
	// Map triggers
	if (KeyDown(diks, PadMap.Trigger[0])) Gamepad.bLeftTrigger = 255;
	if (KeyDown(diks, PadMap.Trigger[1])) Gamepad.bRightTrigger = 255;

	return ERROR_SUCCESS;
}

DWORD WINAPI XInputSetState(DWORD dwUserIndex, XINPUT_VIBRATION* pVibration)
{
	if(!GamepadMapping[dwUserIndex].enabled) return ERROR_DEVICE_NOT_CONNECTED;
	return ERROR_SUCCESS;
}

DWORD WINAPI XInputGetCapabilities(DWORD dwUserIndex, DWORD dwFlags, XINPUT_CAPABILITIES* pCapabilities)
{
	if(!GamepadMapping[dwUserIndex].enabled) return ERROR_DEVICE_NOT_CONNECTED;
	//if(dwFlags != (BYTE)XINPUT_FLAG_GAMEPAD) return ERROR_SUCCESS; //Limit query to devices of Xbox 360 Controller type - Prototype loop fix

	// NOTE: Is this how it's supposed to be set up?
	// Report support for full range and all buttons
	XINPUT_GAMEPAD xGamepad;
	xGamepad.bLeftTrigger = (BYTE)0xFF;
	xGamepad.bRightTrigger = (BYTE)0xFF;
	xGamepad.sThumbLX = (SHORT)0xFFFF;
	xGamepad.sThumbLY = (SHORT)0xFFFF;
	xGamepad.sThumbRX = (SHORT)0xFFFF;
	xGamepad.sThumbRY = (SHORT)0xFFFF;
	xGamepad.wButtons = (WORD)0xFFFFFFFF;

	XINPUT_VIBRATION Vibration = {NULL};

	pCapabilities->Flags = (WORD)XINPUT_FLAG_GAMEPAD;
	pCapabilities->SubType=(BYTE)XINPUT_DEVSUBTYPE_GAMEPAD;
	pCapabilities->Gamepad = xGamepad;
	pCapabilities->Vibration = Vibration;
	pCapabilities->Type = (BYTE)XINPUT_DEVTYPE_GAMEPAD;

	return ERROR_SUCCESS;
}

void WINAPI XInputEnable(BOOL enable)
{
	//TODO
}

DWORD WINAPI XInputGetDSoundAudioDeviceGuids
(
 DWORD dwUserIndex,          // [in] Index of the gamer associated with the device
 GUID* pDSoundRenderGuid,    // [out] DSound device ID for render
 GUID* pDSoundCaptureGuid    // [out] DSound device ID for capture
 )
{
	if(!GamepadMapping[dwUserIndex].enabled) return ERROR_DEVICE_NOT_CONNECTED;
	pDSoundCaptureGuid = pDSoundRenderGuid = NULL;  // FIX: Should be GUID_NULL
	return ERROR_SUCCESS;
}

DWORD WINAPI XInputGetBatteryInformation
(
 DWORD                       dwUserIndex,        // [in]  Index of the gamer associated with the device
 BYTE                        devType,            // [in]  Which device on this user index
 XINPUT_BATTERY_INFORMATION* pBatteryInformation // [out] Contains the level and types of batteries
 )

{
	if(!GamepadMapping[dwUserIndex].enabled) return ERROR_DEVICE_NOT_CONNECTED;

	// Report a wired controller
	pBatteryInformation->BatteryType = BATTERY_TYPE_WIRED;
	return ERROR_SUCCESS;

}

DWORD WINAPI XInputGetKeystroke
(
 DWORD dwUserIndex,              // [in]  Index of the gamer associated with the device
 DWORD dwReserved,               // [in]  Reserved for future use
 PXINPUT_KEYSTROKE pKeystroke    // [out] Pointer to an XINPUT_KEYSTROKE structure that receives an input event.
 )
{
	if(!GamepadMapping[dwUserIndex].enabled) return ERROR_DEVICE_NOT_CONNECTED;
	return ERROR_EMPTY;
}